@@ -1,9 +1,10 @@
 import csv
+import pandas as pd
 
-def read_csv(input_file, quotechar=None):
+def read_csv(input_file, quotechar='"'):
     """Reads a tab separated value file."""
     with open(input_file, "r") as f:
-      reader = csv.reader(f, quotechar=quotechar)
+      reader = csv.reader(f,quotechar=quotechar)
       lines = []
       for line in reader:
         lines.append(line)
