@@ -1,5 +1,5 @@
 from .pretrain import pretrain
-from .evaluate import evaluate, evaluate_ED, eval_predict
+from .evaluate import evaluate, evaluate_ED, eval_predict, write_encode_to_csv, show_retrain_result
 from .adapt_mmd import adapt_mmd
 from .adapt_coral import adapt_coral
 from .adapt_grl import adapt_grl
