@@ -1,11 +1,15 @@
 import torch.nn as nn
 import torch.optim as optim
+import numpy as np
+import copy
+import matplotlib.pyplot as plt
+from matplotlib.image import imread
+from IPython import display
+from tqdm.notebook import tqdm
 import sys
 sys.path.append("..")
 from ..metrics import coral
 from . import evaluate
-import numpy as np
-import copy
 
 def adapt_coral(encoder, matcher, alignment, 
                 src_data_loader, tgt_data_loader, valid_data_loader, 
@@ -20,9 +24,15 @@ def adapt_coral(encoder, matcher, alignment,
     CELoss = nn.CrossEntropyLoss()
     optimizer = optim.Adam(list(encoder.parameters())+list(matcher.parameters()), lr=lr)
     len_data_loader = min(len(src_data_loader), len(tgt_data_loader))
+
+    y = []
+    corals = []
+    clss = []
+    f1_score = []
     
     if valid_data_loader != None:
         bestf1 = 0.0
+        f1_valid = 0.0
         best_encoder = copy.deepcopy(encoder)
         best_matcher = copy.deepcopy(matcher)
         best_alignment = copy.deepcopy(alignment)
@@ -76,7 +86,7 @@ def adapt_coral(encoder, matcher, alignment,
                 loss = cls_loss
             loss.backward()
             optimizer.step()
-
+        
             if (step + 1) % 100 == 0:
                 print("Epoch [%.2d/%.2d] Step [%.3d/%.3d]: "
                       "coral_loss=%.4f cls_loss=%.4f"
@@ -87,6 +97,7 @@ def adapt_coral(encoder, matcher, alignment,
                          loss_coral.item(),
                          cls_loss.item()
                          ))
+        
         if valid_data_loader != None:
             f1_valid = evaluate(encoder, matcher, valid_data_loader, device)
             if f1_valid>bestf1:
